@@ -1,6 +1,5 @@
 #include <iostream>
 #include <string>
-#include <sstream>
 #include <cctype>
 
 #include <lfortran/parser/parser.h>
@@ -122,7 +121,9 @@ Result<AST::TranslationUnit_t*> parse(Allocator &al, const std::string &s,
     Parser p(al, diagnostics, co.fixed_form, co.continue_compilation);
     try {
         if (!p.parse(s)) {
-            return Error();
+            if (!co.continue_compilation) {
+                return Error();
+            }
         };
     } catch (const parser_local::TokenizerError &e) {
         Error error;
@@ -149,7 +150,9 @@ Result<AST::TranslationUnit_t*> parse(Allocator &al, const std::string &s,
         } catch (const parser_local::ParserError &e) {
             Error error;
             diagnostics.diagnostics.push_back(e.d);
-            return error;
+            if (!co.continue_compilation) {
+                return error;
+            }
         }
     }
     return ast;
@@ -237,12 +240,66 @@ Result<std::vector<int>> tokens(Allocator &al, const std::string &input,
     }
 }
 
-void cont1(const std::string &s, size_t &pos, bool &ws_or_comment)
-{
+char previous_nonspace_character(const std::string &s, size_t pos) {
+    while (pos > 0) {
+        --pos;
+        if (s[pos] != ' ' && s[pos] != '\t') {
+            return s[pos];
+        }
+    }
+    return '\0';
+}
+
+char next_nonspace_character(const std::string &s, size_t pos) {
+    pos++;
+    while (pos < s.size()) {
+        if (s[pos] != ' ' && s[pos] != '\t') {
+            return s[pos];
+        }
+        ++pos;
+    }
+    return '\0';
+}
+
+
+void is_within_string(
+    const std::string &s,
+    size_t pos,
+    char &quote,
+    const bool in_comment,
+    bool &in_string
+) {
+    // when in a comment, as everything is ignored, wearen't in string
+    if (in_comment) {
+        return;
+    }
+    if ((s[pos] == '\'' || s[pos] == '"')) {
+        if (quote == '\0') {
+            in_string = true;
+            quote = s[pos];
+        } else if (quote == s[pos]) {
+            in_string = false;
+            quote = '\0';
+        }
+    }
+}
+
+void cont1(
+    const std::string &s,
+    size_t &pos,
+    bool &in_string,
+    char &quote,
+    bool &ws_or_comment
+) {
     ws_or_comment = true;
     bool in_comment = false;
     while (s[pos] != '\n') {
-        if (s[pos] == '!') in_comment = true;
+        // when in a comment, as everything is ignore, wearen't in string
+        is_within_string(s, pos, quote, in_comment, in_string);
+        // in a string if '&!' appear together then it isn't a comment
+        if (s[pos] == '!' && (!in_string || previous_nonspace_character(s, pos) != '&')) {
+            in_comment = true;
+        }
         if (!in_comment) {
             if (s[pos] != ' ' && s[pos] != '\t') {
                 ws_or_comment = false;
@@ -309,7 +366,7 @@ LineType determine_line_type(const unsigned char *pos)
         }
         if (col <= 6) {
             return LineType::LabeledStatement;
-        } else if (std::string(pos, pos + 7) == "include") {
+        } else if (str_compare(pos, "include")) {
             return LineType::Include;
         } else {
             return LineType::Statement;
@@ -506,6 +563,15 @@ bool is_include(const std::string &s, uint32_t pos) {
     }
 }
 
+/*
+The prescan phase includes:
+- Removal of whitespace (fixed-form only)
+- Joining of continuation lines
+- Removal of comments and empty lines
+- Handling of include statements
+- Conversion to lowercase (fixed-form only)
+- Handling of fixed-form column rules (columns 1–6 for labels/comments)
+*/
 std::string prescan(const std::string &s, LocationManager &lm,
         bool fixed_form, std::vector<std::filesystem::path> &include_dirs)
 {
@@ -612,14 +678,20 @@ std::string prescan(const std::string &s, LocationManager &lm,
         lm.files.back().out_start.push_back(out.size());
         return out;
     } else {
-        // `pos` is the position in the original code `s`
+         // `pos` is the position in the original code `s`
         // `out` is the final code (outcome)
         lm.files.back().out_start.push_back(0);
         lm.files.back().in_start.push_back(0);
         std::string out;
         size_t pos = 0;
         bool in_comment = false, newline = true;
+        // keeps track of whether we're in a string or not
+        bool in_string = false;
+        // if `in_string` is true, keeps track of the quote
+        // used for that string
+        char quote = '\0';
         while (pos < s.size()) {
+            is_within_string(s, pos, quote, in_comment, in_string);
             if (newline && is_include(s, pos)) {
                 int col = 0; // doesn't matter
                 while (pos < s.size() && s[pos] == ' ') pos++;
@@ -630,35 +702,36 @@ std::string prescan(const std::string &s, LocationManager &lm,
                 process_include(out, s, lm, pos, fixed_form, include_dirs, col);
             }
             newline = false;
-            if (s[pos] == '!') in_comment = true;
+            if (s[pos] == '!' && !in_string) in_comment = true;
             if (in_comment && s[pos] == '\n') in_comment = false;
-            if (!in_comment && s[pos] == '&') {
+            if (!in_comment && s[pos] == '&' &&(next_nonspace_character(s,pos) == '\n' || next_nonspace_character(s,pos) == '!')) {
                 size_t pos2=pos+1;
-                bool ws_or_comment;
-                cont1(s, pos2, ws_or_comment);
+                bool ws_or_comment = false;
+                cont1(s, pos2, in_string, quote, ws_or_comment);
                 if (ws_or_comment) lm.files.back().in_newlines.push_back(pos2-1);
                 if (ws_or_comment) {
                     while (ws_or_comment) {
-                        cont1(s, pos2, ws_or_comment);
+                        cont1(s, pos2, in_string, quote, ws_or_comment);
                         if (ws_or_comment) lm.files.back().in_newlines.push_back(pos2-1);
-                    }
+                    }}
                     // `pos` will move by more than 1, close the old interval
-    //                lm.in_size.push_back(pos-lm.in_start[lm.in_start.size()-1]);
+                    //lm.in_size.push_back(pos-lm.in_start[lm.in_start.size()-1]);
                     // Move `pos`
                     pos = pos2;
-                    if (s[pos] == '&') pos++;
                     // Start a new interval (just the starts, the size will be
                     // filled in later)
                     lm.files.back().out_start.push_back(out.size());
                     lm.files.back().in_start.push_back(pos);
-                }
+                
             } else {
                 if (s[pos] == '\n') {
                     lm.files.back().in_newlines.push_back(pos);
                     newline = true;
                 }
             }
-            out += s[pos];
+            if (!(s[pos] == '&' &&  previous_nonspace_character(s, pos) == '\n')){
+                out += s[pos];
+            }
             pos++;
         }
         // set the size of the last interval
