@@ -39,7 +39,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
@@ -50,7 +50,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
@@ -61,7 +61,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_or_overwrite_symbol(s2c(al, var_name), sym);
@@ -533,12 +533,26 @@ class ASRBuilder {
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
+                int64_t left_value = 0, right_value = 0;
+                ASR::expr_t* value = nullptr;
+                if( ASRUtils::extract_value(left, left_value) &&
+                    ASRUtils::extract_value(right, right_value) ) {
+                    int64_t mul_value = left_value * right_value;
+                    value = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, mul_value, type));
+                }
                 return EXPR(ASR::make_IntegerBinOp_t(al, loc, left,
-                    ASR::binopType::Mul, right, type, nullptr));
+                    ASR::binopType::Mul, right, type, value));
             }
             case ASR::ttypeType::Real: {
+                double left_value = 0, right_value = 0;
+                ASR::expr_t* value = nullptr;
+                if( ASRUtils::extract_value(left, left_value) &&
+                    ASRUtils::extract_value(right, right_value) ) {
+                    double mul_value = left_value * right_value;
+                    value = ASRUtils::EXPR(ASR::make_RealConstant_t(al, loc, mul_value, type));
+                }
                 return EXPR(ASR::make_RealBinOp_t(al, loc, left,
-                    ASR::binopType::Mul, right, type, nullptr));
+                    ASR::binopType::Mul, right, type, value));
             }
             case ASR::ttypeType::Complex: {
                 return EXPR(ASR::make_ComplexBinOp_t(al, loc, left,
@@ -886,7 +900,8 @@ class ASRBuilder {
     }
 
     ASR::stmt_t *Assignment(ASR::expr_t *lhs, ASR::expr_t *rhs) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(lhs), expr_type(rhs)));
+        LCOMPILERS_ASSERT_MSG(check_equal_type(expr_type(lhs), expr_type(rhs)),
+            type_to_str_python(expr_type(lhs)) + ", " + type_to_str_python(expr_type(rhs)));
         return STMT(ASR::make_Assignment_t(al, loc, lhs, rhs, nullptr));
     }
 
