@@ -186,7 +186,7 @@ intrinsic_funcs_args = {
     "Logical": [
         {
             "args": [("bool", )],
-            "ret_type_arg_idx": 0, 
+            "ret_type_arg_idx": 0,
             "kind_arg": True
         }
     ],
@@ -326,6 +326,12 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Idnint": [
+        {
+            "args": [("real",)],
+            "return": "int32"
+        }
+    ],
     "Anint": [
         {
             "args": [("real",)],
@@ -767,7 +773,7 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0,
             "same_kind_arg": 3
         }
-    ], 
+    ],
     "Ishftc": [
         {
             "args": [("int", "int", "int")],
@@ -848,8 +854,9 @@ intrinsic_funcs_args = {
         {
             "args": [("int",), ("real",), ("complex",)],
             "return": "real32",
-            "kind_arg": True,
-        },
+            "kind_arg": True, 
+            "real_32_except_complex": True
+        },   
     ],
     "Int": [
         {
@@ -1040,12 +1047,12 @@ def add_create_func_return_src(func_name):
             src += indent * 2 + f"for(size_t i=0;i<args.size();i++){{\n"
             src += indent * 3 + f"upper_kind = std::max(upper_kind,ASRUtils::extract_kind_from_ttype_t(expr_type(args[i])));\n"
             src += indent * 2 + f"}}\n"
-            src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(ASRUtils::type_get_past_array_pointer_allocatable(expr_type(args[0])));\n"
+            src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[0])));\n"
             src += indent * 2 + f"set_kind_to_ttype_t(type_,upper_kind);\n"
         else:
             src += indent * 2 + "ASR::ttype_t* type_ = nullptr;\n"
             src += indent * 2 + "if (use_experimental_simplifier) {\n"
-            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::type_get_past_array_pointer_allocatable(expr_type(args[{ret_type_arg_idx}])));\n"
+            src += indent * 2 +     f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
             src += indent * 2 + "} else {\n"
             src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
             src += indent * 2 + "}\n"
@@ -1060,7 +1067,16 @@ def add_create_func_return_src(func_name):
         src += indent * 4 +         "return nullptr;\n"
         src += indent * 3 +     "}\n"
         src += indent * 3 +     "set_kind_to_ttype_t(return_type, kind);\n"
-        src += indent * 2 + "}\n"        
+        src += indent * 2 + "}\n"
+    real_32_except_complex = arg_infos[0].get("real_32_except_complex", False)
+    if real_32_except_complex:
+        src += indent * 2 + "else { \n"
+        src += indent * 3 + "ASR::ttype_t* arg_type = ASRUtils::expr_type(args[0]);\n"
+        src += indent * 3 + "if (is_complex(*arg_type)) { \n"
+        src += indent * 4 + "int kind = ASRUtils::extract_kind_from_ttype_t(arg_type); \n"
+        src += indent * 4 + "set_kind_to_ttype_t(return_type, kind); \n"
+        src += indent * 3 + "} \n"
+        src += indent * 2 + "} \n"
     src += indent * 2 + "ASR::expr_t *m_value = nullptr;\n"
     src += indent * 2 + f"Vec<ASR::expr_t*> m_args; m_args.reserve(al, {no_of_args});\n"
     for _i in range(no_of_args):
@@ -1084,7 +1100,7 @@ def add_create_func_return_src(func_name):
         src += indent * 5 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
         src += indent * 5 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
         src +=                              "return_type, m_dims, n_dims, ASR::abiType::Source, false, "
-        src +=                              "ASR::array_physical_typeType::DescriptorArray, true);\n"
+        src +=                              "ASR::array_physical_typeType::DescriptorArray);\n"
         src += indent * 5 +             "break;\n"
         src += indent * 4 +         "}\n"
         src += indent * 3 +     "}\n"
