@@ -7,8 +7,6 @@
 #include <libasr/pass/pass_utils.h>
 #include <libasr/pass/intrinsic_function_registry.h>
 
-#include <vector>
-#include <utility>
 
 namespace LCompilers {
 
@@ -35,9 +33,9 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         bool realloc_lhs_, bool allocate_target_) :
     al(al_), pass_result(pass_result_),
     remove_original_statement(remove_original_statement_),
-    current_scope(nullptr), 
+    current_scope(nullptr),
     result_var(nullptr), result_counter(0),
-    resultvar2value(resultvar2value_), 
+    resultvar2value(resultvar2value_),
     realloc_lhs(realloc_lhs_), allocate_target(allocate_target_) {}
 
     ASR::expr_t* get_ImpliedDoLoop_size(ASR::ImpliedDoLoop_t* implied_doloop) {
@@ -50,9 +48,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         int kind = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(end));
         start = builder.i2i_t(start, ASRUtils::expr_type(end));
         if( d == nullptr ) {
-            implied_doloop_size = builder.Add(
-                builder.Sub(end, start),
-                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc));
+            implied_doloop_size = ASRUtils::compute_length_from_start_end(al, start, end);
         } else {
             implied_doloop_size = builder.Add(builder.Div(
                 builder.Sub(end, start), d),
@@ -178,6 +174,66 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 } else {
                     array_size = builder.Add(array_section_size, array_size);
                 }
+            } else if( ASR::is_a<ASR::IntrinsicElementalFunction_t>(*element) &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) &&
+                       ASRUtils::use_experimental_simplifier ) {
+                ASR::IntrinsicElementalFunction_t* intrinsic_element_t = ASR::down_cast<ASR::IntrinsicElementalFunction_t>(element);
+                if( ASRUtils::is_fixed_size_array(intrinsic_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(intrinsic_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( ASR::is_a<ASR::FunctionCall_t>(*element)  &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) &&
+                       ASRUtils::use_experimental_simplifier ) {
+                ASR::FunctionCall_t* fc_element_t = ASR::down_cast<ASR::FunctionCall_t>(element);
+                if( ASRUtils::is_fixed_size_array(fc_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(fc_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( ASR::is_a<ASR::IntrinsicArrayFunction_t>(*element)  &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) &&
+                       ASRUtils::use_experimental_simplifier ) {
+                ASR::IntrinsicArrayFunction_t* intrinsic_element_t = ASR::down_cast<ASR::IntrinsicArrayFunction_t>(element);
+                if( ASRUtils::is_fixed_size_array(intrinsic_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(intrinsic_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( ASR::is_a<ASR::RealBinOp_t>(*element) &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) &&
+                       ASRUtils::use_experimental_simplifier ) {
+                ASR::RealBinOp_t* real_binop_t = ASR::down_cast<ASR::RealBinOp_t>(element);
+                if( ASRUtils::is_fixed_size_array(real_binop_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(real_binop_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
             } else {
                 constant_size += 1;
             }
@@ -289,6 +345,14 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 al, loc, alloc_args.p, alloc_args.size(), nullptr, nullptr, nullptr));
             pass_result.push_back(al, allocate_stmt);
         }
+        for (size_t i = 0; i < x->n_args; i++) {
+            if(ASR::is_a<ASR::ArrayItem_t>(*x->m_args[i]) || ASR::is_a<ASR::ArraySection_t>(*x->m_args[i])){
+                ASR::expr_t** temp = current_expr;
+                current_expr = &(x->m_args[i]);
+                self().replace_expr(x->m_args[i]);
+                current_expr = temp;
+            }
+        }
         LCOMPILERS_ASSERT(result_var != nullptr);
         Vec<ASR::stmt_t*>* result_vec = &pass_result;
         if (ASRUtils::use_experimental_simplifier) {
@@ -389,8 +453,12 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
 
     void replace_ArrayPhysicalCast(ASR::ArrayPhysicalCast_t* x) {
         ASR::BaseExprReplacer<ReplaceArrayConstant>::replace_ArrayPhysicalCast(x);
-        // TODO: Allow for DescriptorArray to DescriptorArray physical cast for allocatables
-        // later on
+        if( ASRUtils::use_experimental_simplifier ) {
+            if( x->m_old != ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg)) ) {
+                x->m_old = ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg));
+            }
+        }
+
         if( (x->m_old == x->m_new &&
              x->m_old != ASR::array_physical_typeType::DescriptorArray) ||
             (x->m_old == x->m_new && x->m_old == ASR::array_physical_typeType::DescriptorArray &&
@@ -432,6 +500,10 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
         replacer(al_, pass_result, remove_original_statement,
             resultvar2value, realloc_lhs_, allocate_target),
         parent_body(nullptr) {
+            if( ASRUtils::use_experimental_simplifier ) {
+                call_replacer_on_value = false;
+                replacer.call_replacer_on_value = false;
+            }
             pass_result.n = 0;
             pass_result.reserve(al, 0);
             print = false, file_write = false;
@@ -503,6 +575,8 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
             ASR::expr_t** current_expr_copy_9 = current_expr;
             current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
             this->call_replacer();
+            current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
+            this->call_replacer();
             current_expr = current_expr_copy_9;
             if( !remove_original_statement ) {
                 this->visit_expr(*x.m_value);
@@ -556,7 +630,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                     args.reserve(al, 1);
                     args.push_back(al, x->m_values[i]);
                     ASR::expr_t* fmt_val = ASRUtils::EXPR(ASR::make_StringFormat_t(al,x->base.base.loc, nullptr,
-                        args.p, 1,ASR::string_format_kindType::FormatFortran, 
+                        args.p, 1,ASR::string_format_kindType::FormatFortran,
                         ASRUtils::TYPE(ASR::make_String_t(al,x->base.base.loc,-1,-1,nullptr, ASR::string_physical_typeType::PointerString)), nullptr));
                     print_values.push_back(al, fmt_val);
                     if ( print ) {
