@@ -3,8 +3,9 @@
 
 #include <libasr/asr.h>
 #include <libasr/containers.h>
+#include <libasr/asr_pass_walk_visitor.h>
 
-#include <queue>
+#include <deque>
 
 namespace LCompilers {
 
@@ -92,7 +93,7 @@ namespace LCompilers {
 
         ASR::expr_t* create_auxiliary_variable(const Location& loc, std::string& name,
             Allocator& al, SymbolTable*& current_scope, ASR::ttype_t* var_type,
-            ASR::intentType var_intent=ASR::intentType::Local);
+            ASR::intentType var_intent=ASR::intentType::Local, ASR::symbol_t* var_decl=nullptr);
 
         ASR::expr_t* get_fma(ASR::expr_t* arg0, ASR::expr_t* arg1, ASR::expr_t* arg2,
                              Allocator& al, ASR::TranslationUnit_t& unit, Location& loc,
@@ -153,11 +154,25 @@ namespace LCompilers {
             return ASR::is_a<ASR::StructType_t>(*ASRUtils::expr_type(var));
         }
 
+        /*  Checks for any non-primitive-function-return type 
+            like fixed strings or allocatables.
+            allocatable string, allocatable integer, etc.. */
+        static inline bool is_non_primitive_return_type(ASR::ttype_t* x){
+            // TODO : Handle other allocatable types and fixed strings.
+            return ASRUtils::is_descriptorString(x);
+        }
+
         static inline bool is_aggregate_or_array_type(ASR::expr_t* var) {
             return (ASR::is_a<ASR::StructType_t>(*ASRUtils::expr_type(var)) ||
                     ASRUtils::is_array(ASRUtils::expr_type(var)) ||
                     ASR::is_a<ASR::SymbolicExpression_t>(*ASRUtils::expr_type(var)));
         }
+        
+        static inline bool is_aggregate_or_array_or_nonPrimitive_type(ASR::expr_t* var) {
+            return  is_aggregate_or_array_type(var) || 
+                    is_non_primitive_return_type(ASRUtils::expr_type(var));
+        }
+
 
         static inline bool is_symbolic_list_type(ASR::expr_t* var) {
             if (ASR::is_a<ASR::List_t>(*ASRUtils::expr_type(var))) {
@@ -588,7 +603,7 @@ namespace LCompilers {
                     visit_UserDefinedType(x);
                 }
 
-                void visit_UnionType(const ASR::UnionType_t& x) {
+                void visit_Union(const ASR::Union_t& x) {
                     visit_UserDefinedType(x);
                 }
             */
@@ -609,7 +624,7 @@ namespace LCompilers {
                 return ;
             }
             if( replacer->result_var == nullptr ) {
-                std::string result_var_name = replacer->current_scope->get_unique_name("temp_struct_var__", false);
+                std::string result_var_name = replacer->current_scope->get_unique_name("temp_struct_var__");
                 replacer->result_var = PassUtils::create_auxiliary_variable(x->base.base.loc,
                                     result_var_name, replacer->al, replacer->current_scope, x->m_type);
                 *replacer->current_expr = replacer->result_var;
@@ -681,6 +696,7 @@ namespace LCompilers {
                     result_vec->push_back(replacer->al, assign);
                 }
             }
+            replacer->result_var = nullptr;
         }
 
         static inline void create_do_loop(Allocator& al, ASR::ImpliedDoLoop_t* idoloop,
@@ -794,23 +810,81 @@ namespace LCompilers {
 
         template <typename LOOP_BODY>
         static inline void create_do_loop(Allocator& al, const Location& loc,
-            ASR::ArraySection_t* array_section, Vec<ASR::expr_t*>& idx_vars,
+            ASR::ArrayItem_t* array_item, Vec<ASR::expr_t*>& idx_vars, Vec<ASR::expr_t*>& temp_idx_vars, 
+            Vec<ASR::stmt_t*>& doloop_body, LOOP_BODY loop_body, SymbolTable* current_scope,
+            Vec<ASR::stmt_t*>* result_vec) {
+            int value_rank = array_item->n_args;
+            PassUtils::create_idx_vars(idx_vars, value_rank, loc, al, current_scope, "_t");
+            LCOMPILERS_ASSERT(value_rank == (int) idx_vars.size())
+            temp_idx_vars.reserve(al, array_item->n_args);
+            for( int i = 0; i < value_rank; i++ ) {
+                if(ASRUtils::is_array(ASRUtils::expr_type(array_item->m_args[i].m_right))){
+                    ASR::expr_t* ref = PassUtils::create_array_ref(array_item->m_args[i].m_right, idx_vars[i], al, current_scope);
+                    temp_idx_vars.push_back(al, ref);
+                } else { 
+                    temp_idx_vars.push_back(al, array_item->m_args[i].m_right);
+                }
+            }
+            ASR::stmt_t* doloop = nullptr;
+            for( int i = 0; i < value_rank; i++ ) {
+                ASR::do_loop_head_t head;
+                head.m_v = idx_vars[i];
+                if(ASRUtils::is_array(ASRUtils::expr_type(array_item->m_args[i].m_right))) {
+                    head.m_start = PassUtils::get_bound(array_item->m_args[i].m_right, 1, "lbound", al);
+                    head.m_end = PassUtils::get_bound(array_item->m_args[i].m_right, 1, "ubound", al);
+                    head.m_increment = nullptr;
+                } else {
+                    continue;
+                } 
+                head.loc = head.m_v->base.loc;
+
+                doloop_body.reserve(al, 1);
+                if( doloop == nullptr ) {
+                    loop_body();
+                } else {
+                    doloop_body.push_back(al, doloop);
+                }
+                doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr, head,
+                    doloop_body.p, doloop_body.size(), nullptr, 0));
+            }
+            if(doloop != nullptr) result_vec->push_back(al, doloop);
+        }
+
+        template <typename LOOP_BODY>
+        static inline void create_do_loop(Allocator& al, const Location& loc,
+            ASR::ArraySection_t* array_section, Vec<ASR::expr_t*>& idx_vars, Vec<ASR::expr_t*>& temp_idx_vars,
             Vec<ASR::stmt_t*>& doloop_body, LOOP_BODY loop_body, SymbolTable* current_scope,
             Vec<ASR::stmt_t*>* result_vec) {
             PassUtils::create_idx_vars(idx_vars, array_section->n_args, loc, al, current_scope, "_t");
             LCOMPILERS_ASSERT(array_section->n_args == idx_vars.size())
+            temp_idx_vars.reserve(al, array_section->n_args);
+            for( size_t i = 0; i < array_section->n_args; i++ ) {
+                if ( ASRUtils::is_array(ASRUtils::expr_type(array_section->m_args[i].m_right)) ) {
+                    ASR::expr_t* ref = PassUtils::create_array_ref(array_section->m_args[i].m_right, idx_vars[i], al, current_scope);
+                    temp_idx_vars.push_back(al, ref);
+                } else if ( array_section->m_args[i].m_step != nullptr ) { 
+                    temp_idx_vars.push_back(al, idx_vars[i]);
+                } else {
+                    temp_idx_vars.push_back(al, array_section->m_args[i].m_right);
+                }
+            }
 
             ASR::stmt_t* doloop = nullptr;
             for( size_t i = 0; i < array_section->n_args; i++ ) {
-                if( array_section->m_args[i].m_step == nullptr ) {
-                    continue ;
-                }
                 // TODO: Add an If debug node to check if the lower and upper bounds of both the arrays are same.
                 ASR::do_loop_head_t head;
                 head.m_v = idx_vars[i];
-                head.m_start = array_section->m_args[i].m_left;
-                head.m_end = array_section->m_args[i].m_right;
-                head.m_increment = array_section->m_args[i].m_step;
+                if( ASRUtils::is_array(ASRUtils::expr_type(array_section->m_args[i].m_right)) ) {
+                    head.m_start = PassUtils::get_bound(array_section->m_args[i].m_right, 1, "lbound", al);
+                    head.m_end = PassUtils::get_bound(array_section->m_args[i].m_right, 1, "ubound", al);
+                    head.m_increment = nullptr;
+                } else if ( array_section->m_args[i].m_step == nullptr ) {
+                    continue ;
+                } else {
+                    head.m_start = array_section->m_args[i].m_left;
+                    head.m_end = array_section->m_args[i].m_right;
+                    head.m_increment = array_section->m_args[i].m_step;
+                }
                 head.loc = head.m_v->base.loc;
 
                 doloop_body.reserve(al, 1);
@@ -825,6 +899,116 @@ namespace LCompilers {
             result_vec->push_back(al, doloop);
         }
 
+        template <typename LOOP_BODY>
+        static inline void create_do_loop_assign(Allocator& al, const Location& loc,
+            ASR::expr_t* lhs, ASR::expr_t* rhs, Vec<ASR::expr_t*>& idx_vars, Vec<ASR::expr_t*>& temp_idx_vars_lhs, 
+            Vec<ASR::expr_t*>& temp_idx_vars_rhs, Vec<ASR::stmt_t*>& doloop_body, LOOP_BODY loop_body, 
+            SymbolTable* current_scope, Vec<ASR::stmt_t*>* result_vec) {
+            if (ASR::is_a<ASR::ArrayItem_t>(*lhs) && ASR::is_a<ASR::ArrayItem_t>(*rhs)) {
+                // Case : A([1,2,3]) = B([1,2,3])
+                ASR::ArrayItem_t* lhs_array = ASR::down_cast<ASR::ArrayItem_t>(lhs);
+                ASR::ArrayItem_t* rhs_array = ASR::down_cast<ASR::ArrayItem_t>(rhs);
+                int n_array_indices = 0;
+                for( size_t i = 0; i < rhs_array->n_args; i++ ) {
+                    if (ASRUtils::is_array(ASRUtils::expr_type(rhs_array->m_args[i].m_right))) {
+                        n_array_indices++;
+                    }
+                }
+                if(n_array_indices == 0) return;
+                PassUtils::create_idx_vars(idx_vars, n_array_indices, loc, al, current_scope, "_t");
+                temp_idx_vars_rhs.reserve(al, rhs_array->n_args);
+                temp_idx_vars_lhs.reserve(al, lhs_array->n_args);
+                for( size_t i = 0, j = 0; i < rhs_array->n_args; i++ ) {
+                    if (ASRUtils::is_array(ASRUtils::expr_type(rhs_array->m_args[i].m_right))) {
+                        ASR::expr_t* ref = PassUtils::create_array_ref(rhs_array->m_args[i].m_right, idx_vars[j], al, current_scope);
+                        temp_idx_vars_rhs.push_back(al, ref);
+                        j++;
+                    } else { 
+                        temp_idx_vars_rhs.push_back(al, rhs_array->m_args[i].m_right);
+                    }
+                }
+                for( size_t i = 0, j = 0; i < lhs_array->n_args; i++ ) {
+                    if (ASRUtils::is_array(ASRUtils::expr_type(lhs_array->m_args[i].m_right))) {
+                        ASR::expr_t* ref = PassUtils::create_array_ref(lhs_array->m_args[i].m_right, idx_vars[j], al, current_scope);
+                        temp_idx_vars_lhs.push_back(al, ref);
+                        j++;
+                    } else { 
+                        temp_idx_vars_lhs.push_back(al, lhs_array->m_args[i].m_right);
+                    }
+                }
+
+                ASR::stmt_t* doloop = nullptr;
+                for( size_t i = 0, j = 0; i < rhs_array->n_args; i++ ) {
+                    ASR::do_loop_head_t head;
+                    head.m_v = idx_vars[j];
+                    if (ASRUtils::is_array(ASRUtils::expr_type(rhs_array->m_args[i].m_right))) {
+                        head.m_start = PassUtils::get_bound(rhs_array->m_args[i].m_right, 1, "lbound", al);
+                        head.m_end = PassUtils::get_bound(rhs_array->m_args[i].m_right, 1, "ubound", al);
+                        head.m_increment = nullptr;
+                        j++;
+                    } else {
+                        continue;
+                    } 
+                    head.loc = head.m_v->base.loc;
+
+                    doloop_body.reserve(al, 1);
+                    if( doloop == nullptr ) {
+                        loop_body();
+                    } else {
+                        doloop_body.push_back(al, doloop);
+                    }
+                    doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr, head,
+                        doloop_body.p, doloop_body.size(), nullptr, 0));
+                }
+                result_vec->push_back(al, doloop);
+            } else if (ASR::is_a<ASR::ArrayItem_t>(*lhs) && ASR::is_a<ASR::Var_t>(*rhs)) {
+                // Case : A([1,2,3]) = A2
+                ASR::ArrayItem_t* lhs_array = ASR::down_cast<ASR::ArrayItem_t>(lhs);
+                int n_array_indices = ASRUtils::extract_n_dims_from_ttype(ASRUtils::expr_type(rhs));
+                if(n_array_indices == 0) return;
+                PassUtils::create_idx_vars(idx_vars, n_array_indices, loc, al, current_scope, "_t");
+                temp_idx_vars_rhs.reserve(al, n_array_indices);
+                temp_idx_vars_lhs.reserve(al, lhs_array->n_args);
+                for( int i = 0; i < n_array_indices; i++ ) {
+                    temp_idx_vars_rhs.push_back(al, idx_vars[i]);
+                }
+                for( size_t i = 0, j = 0; i < lhs_array->n_args; i++ ) {
+                    if (ASRUtils::is_array(ASRUtils::expr_type(lhs_array->m_args[i].m_right))) {
+                        ASR::expr_t* ref = PassUtils::create_array_ref(lhs_array->m_args[i].m_right, idx_vars[j], al, current_scope);
+                        temp_idx_vars_lhs.push_back(al, ref);
+                        j++;
+                    } else { 
+                        temp_idx_vars_lhs.push_back(al, lhs_array->m_args[i].m_right);
+                    }
+                }
+
+                ASR::stmt_t* doloop = nullptr;
+                for( size_t i = 0, j = 0; i < lhs_array->n_args; i++ ) {
+                    ASR::do_loop_head_t head;
+                    head.m_v = idx_vars[j];
+                    if (ASRUtils::is_array(ASRUtils::expr_type(lhs_array->m_args[i].m_right))) {
+                        head.m_start = PassUtils::get_bound(lhs_array->m_args[i].m_right, 1, "lbound", al);
+                        head.m_end = PassUtils::get_bound(lhs_array->m_args[i].m_right, 1, "ubound", al);
+                        head.m_increment = nullptr;
+                        j++;
+                    } else {
+                        continue;
+                    } 
+                    head.loc = head.m_v->base.loc;
+
+                    doloop_body.reserve(al, 1);
+                    if( doloop == nullptr ) {
+                        loop_body();
+                    } else {
+                        doloop_body.push_back(al, doloop);
+                    }
+                    doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr, head,
+                        doloop_body.p, doloop_body.size(), nullptr, 0));
+                }
+                result_vec->push_back(al, doloop);
+            }
+        }
+
         void visit_ArrayConstant(ASR::ArrayConstant_t* x, Allocator& al,
             ASR::expr_t* arr_var, Vec<ASR::stmt_t*>* result_vec,
             ASR::expr_t* idx_var, SymbolTable* current_scope,
